/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: Matthew Whyte, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

package org.greenstone.gatherer.util;

import java.io.File;

/*
  Class to zip up a given directory of a given Greenstone collection.

  Arguments:
  gsdlhome, the home of greensone, eg "/research/mgw5/gsdl/"
  collecion, thename of the collection, eg "mgw5-test"
  dir, the name of the directory to zip up, eg "archives"
  [accept_expression], a regular expression of files to exclusively include in the archive
  [reject_expression], a regular expression of files to exclude from the archive

  @see org.greenstone.gatherer.util.Utility#zipup
 */
public class Zipup {

    public static void main(String[] args) {

	
	if (args.length < 3 || args.length > 8) {
	    System.err.println("Usage: Zipup gsdlhome collection directory [-accept accept_expression] [-reject reject_expression]");
	    return;
	}
	
	boolean metadata_only = false;
	String gsdl_home = args[0];
	String collection = args[1];
	String directory = args[2];
	String accept_expr = "";
	String reject_expr = "";

	for(int i=0; i < args.length; i++) {
	    if(args[i].compareTo("-accept") == 0) {
		if(args.length >= i+1) {
		    accept_expr = args[++i];
		}
		else {
		    System.err.println("Usage: Zipup gsdlhome collection directory [-accept accept_expression] [-reject reject_expression]");
		    return;
		}
	    }
	    else if(args[i].compareTo("-reject") == 0) {
		if(args.length >= i+1) {
		    reject_expr = args[++i];
		}
		else {
		    System.err.println("Usage: Zipup gsdlhome collection directory [-accept accept_expression] [-reject reject_expression]");
		    return;
		}
	    }
	}     
	       
	Utility.zipup(gsdl_home + "collect/", collection, directory, null, accept_expr, reject_expr);
    }
}
